# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


"""Tests for tunings.list()."""

import pytest

from ... import types as genai_types
from .. import pytest_helper


test_table: list[pytest_helper.TestTableItem] = [
    pytest_helper.TestTableItem(
        name='test_default',
        parameters=genai_types._ListTuningJobsParameters(),
    ),
    pytest_helper.TestTableItem(
        name='test_with_config',
        parameters=genai_types._ListTuningJobsParameters(
            config=genai_types.ListTuningJobsConfig(page_size=2)
        ),
    ),
]

pytestmark = pytest_helper.setup(
    file=__file__,
    globals_for_file=globals(),
    test_method='tunings.list',
    test_table=test_table,
)

pytest_plugins = ('pytest_asyncio',)


def test_pager(client):
  tuning_jobs = client.tunings.list(config={'page_size': 2})
  assert 'content-type' in tuning_jobs.sdk_http_response.headers
  assert tuning_jobs.name == 'tuning_jobs'
  assert tuning_jobs.page_size == 2
  assert len(tuning_jobs) <= 2

  # Iterate through all the pages. Then next_page() should raise an exception.
  for _ in tuning_jobs:
    pass
  with pytest.raises(IndexError, match='No more pages to fetch.'):
    tuning_jobs.next_page()


@pytest.mark.asyncio
async def test_async_pager(client):
  tuning_jobs = await client.aio.tunings.list(config={'page_size': 2})

  assert 'Content-Type' in tuning_jobs.sdk_http_response.headers
  assert tuning_jobs.name == 'tuning_jobs'
  assert tuning_jobs.page_size == 2
  assert len(tuning_jobs) <= 2

  # Iterate through all the pages. Then next_page() should raise an exception.
  async for _ in tuning_jobs:
    pass
  with pytest.raises(IndexError, match='No more pages to fetch.'):
    await tuning_jobs.next_page()



# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


"""Tests for live.py."""
from ... import types
from ... import _transformers as t

def test_function_response_dict():
  input = {
      'name': 'get_current_weather',
      'response': {'temperature': 14.5, 'unit': 'C'},
      'id': 'some-id',
  }

  function_responses = t.t_function_responses(input)

  assert len(function_responses) == 1
  assert function_responses[0].name == 'get_current_weather'
  assert function_responses[0].response['temperature'] == 14.5
  assert function_responses[0].response['unit'] == 'C'


def test_send_function_response():
  input = types.FunctionResponse(
      name='get_current_weather',
      response={'temperature': 14.5, 'unit': 'C'},
      id='some-id',
  )
  
  function_responses = t.t_function_responses(input)

  assert len(function_responses) == 1
  assert function_responses[0].name == 'get_current_weather'
  assert function_responses[0].response['temperature'] == 14.5
  assert function_responses[0].response['unit'] == 'C'


def test_send_function_response_list():

  input1 = {
      'name': 'get_current_weather',
      'response': {'temperature': 14.5, 'unit': 'C'},
      'id': '1',
  }
  input2 = {
      'name': 'get_current_weather',
      'response': {'temperature': 99.9, 'unit': 'C'},
      'id': '2',
  }

  function_responses = t.t_function_responses([input1, input2])

  assert len(function_responses) == 2
  assert function_responses[0].name == 'get_current_weather'
  assert function_responses[0].response['temperature'] == 14.5
  assert function_responses[0].response['unit'] == 'C'
  assert function_responses[1].name == 'get_current_weather'
  assert function_responses[1].response['temperature'] == 99.9
  assert function_responses[1].response['unit'] == 'C'



# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Union, Iterable
from typing_extensions import Literal, overload

import httpx

from ..types import interaction_get_params, interaction_create_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import required_args, maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._streaming import Stream, AsyncStream
from .._base_client import make_request_options
from ..types.tool_param import ToolParam
from ..types.interaction import Interaction
from ..types.model_param import ModelParam
from ..types.interaction_sse_event import InteractionSSEEvent
from ..types.generation_config_param import GenerationConfigParam

__all__ = ["InteractionsResource", "AsyncInteractionsResource"]


class InteractionsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> InteractionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/stainless-sdks/gemini-next-gen-api-python#accessing-raw-response-data-eg-headers
        """
        return InteractionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> InteractionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/stainless-sdks/gemini-next-gen-api-python#with_streaming_response
        """
        return InteractionsResourceWithStreamingResponse(self)

    @overload
    def create(
        self,
        *,
        api_version: str | None = None,
        input: interaction_create_params.Input,
        model: ModelParam,
        background: bool | Omit = omit,
        generation_config: GenerationConfigParam | Omit = omit,
        previous_interaction_id: str | Omit = omit,
        response_format: object | Omit = omit,
        response_mime_type: str | Omit = omit,
        response_modalities: List[Literal["text", "image", "audio"]] | Omit = omit,
        service_tier: Literal["flex", "standard", "priority"] | Omit = omit,
        store: bool | Omit = omit,
        stream: Literal[False] | Omit = omit,
        system_instruction: str | Omit = omit,
        tools: Iterable[ToolParam] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Interaction:
        """
        Creates a new interaction.

        Args:
          input: The input for the interaction.

          model: The name of the `Model` used for generating the interaction.

          background: Input only. Whether to run the model interaction in the background.

          generation_config: Input only. Configuration parameters for the model interaction.

          previous_interaction_id: The ID of the previous interaction, if any.

          response_format: Enforces that the generated response is a JSON object that complies with
              the JSON schema specified in this field.

          response_mime_type: The mime type of the response. This is required if response_format is set.

          response_modalities: The requested modalities of the response (TEXT, IMAGE, AUDIO).

          service_tier: The service tier for the interaction.

          store: Input only. Whether to store the response and request for later retrieval.

          stream: Input only. Whether the interaction will be streamed.

          system_instruction: System instruction for the interaction.

          tools: A list of tool declarations the model may call during interaction.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def create(
        self,
        *,
        api_version: str | None = None,
        input: interaction_create_params.Input,
        model: ModelParam,
        stream: Literal[True],
        background: bool | Omit = omit,
        generation_config: GenerationConfigParam | Omit = omit,
        previous_interaction_id: str | Omit = omit,
        response_format: object | Omit = omit,
        response_mime_type: str | Omit = omit,
        response_modalities: List[Literal["text", "image", "audio"]] | Omit = omit,
        service_tier: Literal["flex", "standard", "priority"] | Omit = omit,
        store: bool | Omit = omit,
        system_instruction: str | Omit = omit,
        tools: Iterable[ToolParam] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Stream[InteractionSSEEvent]:
        """
        Creates a new interaction.

        Args:
          input: The input for the interaction.

          model: The name of the `Model` used for generating the interaction.

          stream: Input only. Whether the interaction will be streamed.

          background: Input only. Whether to run the model interaction in the background.

          generation_config: Input only. Configuration parameters for the model interaction.

          previous_interaction_id: The ID of the previous interaction, if any.

          response_format: Enforces that the generated response is a JSON object that complies with
              the JSON schema specified in this field.

          response_mime_type: The mime type of the response. This is required if response_format is set.

          response_modalities: The requested modalities of the response (TEXT, IMAGE, AUDIO).

          service_tier: The service tier for the interaction.

          store: Input only. Whether to store the response and request for later retrieval.

          system_instruction: System instruction for the interaction.

          tools: A list of tool declarations the model may call during interaction.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def create(
        self,
        *,
        api_version: str | None = None,
        agent: Union[str, Literal["deep-research-pro-preview-12-2025"]],
        input: interaction_create_params.Input,
        agent_config: interaction_create_params.AgentConfig | Omit = omit,
        background: bool | Omit = omit,
        previous_interaction_id: str | Omit = omit,
        response_format: object | Omit = omit,
        response_mime_type: str | Omit = omit,
        response_modalities: List[Literal["text", "image", "audio"]] | Omit = omit,
        service_tier: Literal["flex", "standard", "priority"] | Omit = omit,
        store: bool | Omit = omit,
        stream: Literal[False] | Omit = omit,
        system_instruction: str | Omit = omit,
        tools: Iterable[ToolParam] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Interaction:
        """
        Creates a new interaction.

        Args:
          agent: The name of the `Agent` used for generating the interaction.

          input: The input for the interaction.

          agent_config: Configuration parameters for the agent interaction.

          background: Input only. Whether to run the model interaction in the background.

          previous_interaction_id: The ID of the previous interaction, if any.

          response_format: Enforces that the generated response is a JSON object that complies with
              the JSON schema specified in this field.

          response_mime_type: The mime type of the response. This is required if response_format is set.

          response_modalities: The requested modalities of the response (TEXT, IMAGE, AUDIO).

          service_tier: The service tier for the interaction.

          store: Input only. Whether to store the response and request for later retrieval.

          stream: Input only. Whether the interaction will be streamed.

          system_instruction: System instruction for the interaction.

          tools: A list of tool declarations the model may call during interaction.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def create(
        self,
        *,
        api_version: str | None = None,
        agent: Union[str, Literal["deep-research-pro-preview-12-2025"]],
        input: interaction_create_params.Input,
        stream: Literal[True],
        agent_config: interaction_create_params.AgentConfig | Omit = omit,
        background: bool | Omit = omit,
        previous_interaction_id: str | Omit = omit,
        response_format: object | Omit = omit,
        response_mime_type: str | Omit = omit,
        response_modalities: List[Literal["text", "image", "audio"]] | Omit = omit,
        service_tier: Literal["flex", "standard", "priority"] | Omit = omit,
        store: bool | Omit = omit,
        system_instruction: str | Omit = omit,
        tools: Iterable[ToolParam] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Stream[InteractionSSEEvent]:
        """
        Creates a new interaction.

        Args:
          agent: The name of the `Agent` used for generating the interaction.

          input: The input for the interaction.

          stream: Input only. Whether the interaction will be streamed.

          agent_config: Configuration parameters for the agent interaction.

          background: Input only. Whether to run the model interaction in the background.

          previous_interaction_id: The ID of the previous interaction, if any.

          response_format: Enforces that the generated response is a JSON object that complies with
              the JSON schema specified in this field.

          response_mime_type: The mime type of the response. This is required if response_format is set.

          response_modalities: The requested modalities of the response (TEXT, IMAGE, AUDIO).

          service_tier: The service tier for the interaction.

          store: Input only. Whether to store the response and request for later retrieval.

          system_instruction: System instruction for the interaction.

          tools: A list of tool declarations the model may call during interaction.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def create(
        self,
        *,
        api_version: str | None = None,
        input: interaction_create_params.Input,
        model: ModelParam,
        stream: bool,
        background: bool | Omit = omit,
        generation_config: GenerationConfigParam | Omit = omit,
        previous_interaction_id: str | Omit = omit,
        response_format: object | Omit = omit,
        response_mime_type: str | Omit = omit,
        response_modalities: List[Literal["text", "image", "audio"]] | Omit = omit,
        service_tier: Literal["flex", "standard", "priority"] | Omit = omit,
        store: bool | Omit = omit,
        system_instruction: str | Omit = omit,
        tools: Iterable[ToolParam] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Interaction | Stream[InteractionSSEEvent]:
        """
        Creates a new interaction.

        Args:
          input: The input for the interaction.

          model: The name of the `Model` used for generating the interaction.

          stream: Input only. Whether the interaction will be streamed.

          background: Input only. Whether to run the model interaction in the background.

          generation_config: Input only. Configuration parameters for the model interaction.

          previous_interaction_id: The ID of the previous interaction, if any.

          response_format: Enforces that the generated response is a JSON object that complies with
              the JSON schema specified in this field.

          response_mime_type: The mime type of the response. This is required if response_format is set.

          response_modalities: The requested modalities of the response (TEXT, IMAGE, AUDIO).

          service_tier: The service tier for the interaction.

          store: Input only. Whether to store the response and request for later retrieval.

          system_instruction: System instruction for the interaction.

          tools: A list of tool declarations the model may call during interaction.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(["input", "model"], ["input", "model", "stream"], ["agent", "input"], ["agent", "input", "stream"])
    def create(
        self,
        *,
        api_version: str | None = None,
        input: interaction_create_params.Input,
        model: ModelParam | Omit = omit,
        background: bool | Omit = omit,
        generation_config: GenerationConfigParam | Omit = omit,
        previous_interaction_id: str | Omit = omit,
        response_format: object | Omit = omit,
        response_mime_type: str | Omit = omit,
        response_modalities: List[Literal["text", "image", "audio"]] | Omit = omit,
        service_tier: Literal["flex", "standard", "priority"] | Omit = omit,
        store: bool | Omit = omit,
        stream: Literal[False] | Literal[True] | Omit = omit,
        system_instruction: str | Omit = omit,
        tools: Iterable[ToolParam] | Omit = omit,
        agent: Union[str, Literal["deep-research-pro-preview-12-2025"]] | Omit = omit,
        agent_config: interaction_create_params.AgentConfig | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Interaction | Stream[InteractionSSEEvent]:
        if api_version is None:
            api_version = self._client._get_api_version_path_param()
        if not api_version:
            raise ValueError(f"Expected a non-empty value for `api_version` but received {api_version!r}")
        if model is not omit and agent_config is not omit:
            raise ValueError("Invalid request: specified `model` and `agent_config`. If specifying `model`, use `generation_config`.")
        if agent is not omit and generation_config is not omit:
            raise ValueError("Invalid request: specified `agent` and `generation_config`. If specifying `agent`, use `agent_config`.")
        return self._post(
            self._client._build_maybe_vertex_path(api_version=api_version, path='interactions'),
            body=maybe_transform(
                {
                    "input": input,
                    "model": model,
                    "background": background,
                    "generation_config": generation_config,
                    "previous_interaction_id": previous_interaction_id,
                    "response_format": response_format,
                    "response_mime_type": response_mime_type,
                    "response_modalities": response_modalities,
                    "service_tier": service_tier,
                    "store": store,
                    "stream": stream,
                    "system_instruction": system_instruction,
                    "tools": tools,
                    "agent": agent,
                    "agent_config": agent_config,
                },
                interaction_create_params.CreateModelInteractionParamsStreaming
                if stream
                else interaction_create_params.CreateModelInteractionParamsNonStreaming,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Interaction,
            stream=stream or False,
            stream_cls=Stream[InteractionSSEEvent],
        )

    def delete(
        self,
        id: str,
        *,
        api_version: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Deletes the interaction by id.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if api_version is None:
            api_version = self._client._get_api_version_path_param()
        if not api_version:
            raise ValueError(f"Expected a non-empty value for `api_version` but received {api_version!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            self._client._build_maybe_vertex_path(api_version=api_version, path=f'interactions/{id}'),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    def cancel(
        self,
        id: str,
        *,
        api_version: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Interaction:
        """Cancels an interaction by id.

        This only applies to background interactions that are still running.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if api_version is None:
            api_version = self._client._get_api_version_path_param()
        if not api_version:
            raise ValueError(f"Expected a non-empty value for `api_version` but received {api_version!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._post(
            self._client._build_maybe_vertex_path(api_version=api_version, path=f'interactions/{id}/cancel'),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Interaction,
        )

    @overload
    def get(
        self,
        id: str,
        *,
        api_version: str | None = None,
        include_input: bool | Omit = omit,
        last_event_id: str | Omit = omit,
        stream: Literal[False] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Interaction:
        """
        Retrieves the full details of a single interaction based on its `Interaction.id`.

        Args:
          include_input: If set to true, includes the input in the response.

          last_event_id: Optional. If set, resumes the interaction stream from the next chunk after the event marked by the event id. Can only be used if `stream` is true.

          stream: If set to true, the generated content will be streamed incrementally.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def get(
        self,
        id: str,
        *,
        api_version: str | None = None,
        stream: Literal[True],
        include_input: bool | Omit = omit,
        last_event_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Stream[InteractionSSEEvent]:
        """
        Retrieves the full details of a single interaction based on its `Interaction.id`.

        Args:
          stream: If set to true, the generated content will be streamed incrementally.

          include_input: If set to true, includes the input in the response.

          last_event_id: Optional. If set, resumes the interaction stream from the next chunk after the event marked by the event id. Can only be used if `stream` is true.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def get(
        self,
        id: str,
        *,
        api_version: str | None = None,
        stream: bool,
        include_input: bool | Omit = omit,
        last_event_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Interaction | Stream[InteractionSSEEvent]:
        """
        Retrieves the full details of a single interaction based on its `Interaction.id`.

        Args:
          stream: If set to true, the generated content will be streamed incrementally.

          include_input: If set to true, includes the input in the response.

          last_event_id: Optional. If set, resumes the interaction stream from the next chunk after the event marked by the event id. Can only be used if `stream` is true.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    def get(
        self,
        id: str,
        *,
        api_version: str | None = None,
        include_input: bool | Omit = omit,
        last_event_id: str | Omit = omit,
        stream: Literal[False] | Literal[True] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Interaction | Stream[InteractionSSEEvent]:
        if api_version is None:
            api_version = self._client._get_api_version_path_param()
        if not api_version:
            raise ValueError(f"Expected a non-empty value for `api_version` but received {api_version!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            self._client._build_maybe_vertex_path(api_version=api_version, path=f'interactions/{id}'),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "include_input": include_input,
                        "last_event_id": last_event_id,
                        "stream": stream,
                    },
                    interaction_get_params.InteractionGetParams,
                ),
            ),
            cast_to=Interaction,
            stream=stream or False,
            stream_cls=Stream[InteractionSSEEvent],
        )


class AsyncInteractionsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncInteractionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/stainless-sdks/gemini-next-gen-api-python#accessing-raw-response-data-eg-headers
        """
        return AsyncInteractionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncInteractionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/stainless-sdks/gemini-next-gen-api-python#with_streaming_response
        """
        return AsyncInteractionsResourceWithStreamingResponse(self)

    @overload
    async def create(
        self,
        *,
        api_version: str | None = None,
        input: interaction_create_params.Input,
        model: ModelParam,
        background: bool | Omit = omit,
        generation_config: GenerationConfigParam | Omit = omit,
        previous_interaction_id: str | Omit = omit,
        response_format: object | Omit = omit,
        response_mime_type: str | Omit = omit,
        response_modalities: List[Literal["text", "image", "audio"]] | Omit = omit,
        service_tier: Literal["flex", "standard", "priority"] | Omit = omit,
        store: bool | Omit = omit,
        stream: Literal[False] | Omit = omit,
        system_instruction: str | Omit = omit,
        tools: Iterable[ToolParam] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Interaction:
        """
        Creates a new interaction.

        Args:
          input: The input for the interaction.

          model: The name of the `Model` used for generating the interaction.

          background: Input only. Whether to run the model interaction in the background.

          generation_config: Input only. Configuration parameters for the model interaction.

          previous_interaction_id: The ID of the previous interaction, if any.

          response_format: Enforces that the generated response is a JSON object that complies with
              the JSON schema specified in this field.

          response_mime_type: The mime type of the response. This is required if response_format is set.

          response_modalities: The requested modalities of the response (TEXT, IMAGE, AUDIO).

          service_tier: The service tier for the interaction.

          store: Input only. Whether to store the response and request for later retrieval.

          stream: Input only. Whether the interaction will be streamed.

          system_instruction: System instruction for the interaction.

          tools: A list of tool declarations the model may call during interaction.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def create(
        self,
        *,
        api_version: str | None = None,
        input: interaction_create_params.Input,
        model: ModelParam,
        stream: Literal[True],
        background: bool | Omit = omit,
        generation_config: GenerationConfigParam | Omit = omit,
        previous_interaction_id: str | Omit = omit,
        response_format: object | Omit = omit,
        response_mime_type: str | Omit = omit,
        response_modalities: List[Literal["text", "image", "audio"]] | Omit = omit,
        service_tier: Literal["flex", "standard", "priority"] | Omit = omit,
        store: bool | Omit = omit,
        system_instruction: str | Omit = omit,
        tools: Iterable[ToolParam] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncStream[InteractionSSEEvent]:
        """
        Creates a new interaction.

        Args:
          input: The input for the interaction.

          model: The name of the `Model` used for generating the interaction.

          stream: Input only. Whether the interaction will be streamed.

          background: Input only. Whether to run the model interaction in the background.

          generation_config: Input only. Configuration parameters for the model interaction.

          previous_interaction_id: The ID of the previous interaction, if any.

          response_format: Enforces that the generated response is a JSON object that complies with
              the JSON schema specified in this field.

          response_mime_type: The mime type of the response. This is required if response_format is set.

          response_modalities: The requested modalities of the response (TEXT, IMAGE, AUDIO).

          service_tier: The service tier for the interaction.

          store: Input only. Whether to store the response and request for later retrieval.

          system_instruction: System instruction for the interaction.

          tools: A list of tool declarations the model may call during interaction.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def create(
        self,
        *,
        api_version: str | None = None,
        agent: Union[str, Literal["deep-research-pro-preview-12-2025"]],
        input: interaction_create_params.Input,
        agent_config: interaction_create_params.AgentConfig | Omit = omit,
        background: bool | Omit = omit,
        previous_interaction_id: str | Omit = omit,
        response_format: object | Omit = omit,
        response_mime_type: str | Omit = omit,
        response_modalities: List[Literal["text", "image", "audio"]] | Omit = omit,
        service_tier: Literal["flex", "standard", "priority"] | Omit = omit,
        store: bool | Omit = omit,
        stream: Literal[False] | Omit = omit,
        system_instruction: str | Omit = omit,
        tools: Iterable[ToolParam] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Interaction:
        """
        Creates a new interaction.

        Args:
          agent: The name of the `Agent` used for generating the interaction.

          input: The input for the interaction.

          agent_config: Configuration parameters for the agent interaction.

          background: Input only. Whether to run the model interaction in the background.

          previous_interaction_id: The ID of the previous interaction, if any.

          response_format: Enforces that the generated response is a JSON object that complies with
              the JSON schema specified in this field.

          response_mime_type: The mime type of the response. This is required if response_format is set.

          response_modalities: The requested modalities of the response (TEXT, IMAGE, AUDIO).

          service_tier: The service tier for the interaction.

          store: Input only. Whether to store the response and request for later retrieval.

          stream: Input only. Whether the interaction will be streamed.

          system_instruction: System instruction for the interaction.

          tools: A list of tool declarations the model may call during interaction.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def create(
        self,
        *,
        api_version: str | None = None,
        agent: Union[str, Literal["deep-research-pro-preview-12-2025"]],
        input: interaction_create_params.Input,
        stream: Literal[True],
        agent_config: interaction_create_params.AgentConfig | Omit = omit,
        background: bool | Omit = omit,
        previous_interaction_id: str | Omit = omit,
        response_format: object | Omit = omit,
        response_mime_type: str | Omit = omit,
        response_modalities: List[Literal["text", "image", "audio"]] | Omit = omit,
        service_tier: Literal["flex", "standard", "priority"] | Omit = omit,
        store: bool | Omit = omit,
        system_instruction: str | Omit = omit,
        tools: Iterable[ToolParam] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncStream[InteractionSSEEvent]:
        """
        Creates a new interaction.

        Args:
          agent: The name of the `Agent` used for generating the interaction.

          input: The input for the interaction.

          stream: Input only. Whether the interaction will be streamed.

          agent_config: Configuration parameters for the agent interaction.

          background: Input only. Whether to run the model interaction in the background.

          previous_interaction_id: The ID of the previous interaction, if any.

          response_format: Enforces that the generated response is a JSON object that complies with
              the JSON schema specified in this field.

          response_mime_type: The mime type of the response. This is required if response_format is set.

          response_modalities: The requested modalities of the response (TEXT, IMAGE, AUDIO).

          service_tier: The service tier for the interaction.

          store: Input only. Whether to store the response and request for later retrieval.

          system_instruction: System instruction for the interaction.

          tools: A list of tool declarations the model may call during interaction.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def create(
        self,
        *,
        api_version: str | None = None,
        input: interaction_create_params.Input,
        model: ModelParam,
        stream: bool,
        background: bool | Omit = omit,
        generation_config: GenerationConfigParam | Omit = omit,
        previous_interaction_id: str | Omit = omit,
        response_format: object | Omit = omit,
        response_mime_type: str | Omit = omit,
        response_modalities: List[Literal["text", "image", "audio"]] | Omit = omit,
        service_tier: Literal["flex", "standard", "priority"] | Omit = omit,
        store: bool | Omit = omit,
        system_instruction: str | Omit = omit,
        tools: Iterable[ToolParam] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Interaction | AsyncStream[InteractionSSEEvent]:
        """
        Creates a new interaction.

        Args:
          input: The input for the interaction.

          model: The name of the `Model` used for generating the interaction.

          stream: Input only. Whether the interaction will be streamed.

          background: Input only. Whether to run the model interaction in the background.

          generation_config: Input only. Configuration parameters for the model interaction.

          previous_interaction_id: The ID of the previous interaction, if any.

          response_format: Enforces that the generated response is a JSON object that complies with
              the JSON schema specified in this field.

          response_mime_type: The mime type of the response. This is required if response_format is set.

          response_modalities: The requested modalities of the response (TEXT, IMAGE, AUDIO).

          service_tier: The service tier for the interaction.

          store: Input only. Whether to store the response and request for later retrieval.

          system_instruction: System instruction for the interaction.

          tools: A list of tool declarations the model may call during interaction.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(["input", "model"], ["input", "model", "stream"], ["agent", "input"], ["agent", "input", "stream"])
    async def create(
        self,
        *,
        api_version: str | None = None,
        input: interaction_create_params.Input,
        model: ModelParam | Omit = omit,
        background: bool | Omit = omit,
        generation_config: GenerationConfigParam | Omit = omit,
        previous_interaction_id: str | Omit = omit,
        response_format: object | Omit = omit,
        response_mime_type: str | Omit = omit,
        response_modalities: List[Literal["text", "image", "audio"]] | Omit = omit,
        service_tier: Literal["flex", "standard", "priority"] | Omit = omit,
        store: bool | Omit = omit,
        stream: Literal[False] | Literal[True] | Omit = omit,
        system_instruction: str | Omit = omit,
        tools: Iterable[ToolParam] | Omit = omit,
        agent: Union[str, Literal["deep-research-pro-preview-12-2025"]] | Omit = omit,
        agent_config: interaction_create_params.AgentConfig | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Interaction | AsyncStream[InteractionSSEEvent]:
        if api_version is None:
            api_version = self._client._get_api_version_path_param()
        if not api_version:
            raise ValueError(f"Expected a non-empty value for `api_version` but received {api_version!r}")
        if model is not omit and agent_config is not omit:
            raise ValueError("Invalid request: specified `model` and `agent_config`. If specifying `model`, use `generation_config`.")
        if agent is not omit and generation_config is not omit:
            raise ValueError("Invalid request: specified `agent` and `generation_config`. If specifying `agent`, use `agent_config`.")
        return await self._post(
            self._client._build_maybe_vertex_path(api_version=api_version, path='interactions'),
            body=await async_maybe_transform(
                {
                    "input": input,
                    "model": model,
                    "background": background,
                    "generation_config": generation_config,
                    "previous_interaction_id": previous_interaction_id,
                    "response_format": response_format,
                    "response_mime_type": response_mime_type,
                    "response_modalities": response_modalities,
                    "service_tier": service_tier,
                    "store": store,
                    "stream": stream,
                    "system_instruction": system_instruction,
                    "tools": tools,
                    "agent": agent,
                    "agent_config": agent_config,
                },
                interaction_create_params.CreateModelInteractionParamsStreaming
                if stream
                else interaction_create_params.CreateModelInteractionParamsNonStreaming,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Interaction,
            stream=stream or False,
            stream_cls=AsyncStream[InteractionSSEEvent],
        )

    async def delete(
        self,
        id: str,
        *,
        api_version: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Deletes the interaction by id.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if api_version is None:
            api_version = self._client._get_api_version_path_param()
        if not api_version:
            raise ValueError(f"Expected a non-empty value for `api_version` but received {api_version!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            self._client._build_maybe_vertex_path(api_version=api_version, path=f'interactions/{id}'),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    async def cancel(
        self,
        id: str,
        *,
        api_version: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Interaction:
        """Cancels an interaction by id.

        This only applies to background interactions that are still running.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if api_version is None:
            api_version = self._client._get_api_version_path_param()
        if not api_version:
            raise ValueError(f"Expected a non-empty value for `api_version` but received {api_version!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._post(
            self._client._build_maybe_vertex_path(api_version=api_version, path=f'interactions/{id}/cancel'),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Interaction,
        )

    @overload
    async def get(
        self,
        id: str,
        *,
        api_version: str | None = None,
        include_input: bool | Omit = omit,
        last_event_id: str | Omit = omit,
        stream: Literal[False] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Interaction:
        """
        Retrieves the full details of a single interaction based on its `Interaction.id`.

        Args:
          include_input: If set to true, includes the input in the response.

          last_event_id: Optional. If set, resumes the interaction stream from the next chunk after the event marked by the event id. Can only be used if `stream` is true.

          stream: If set to true, the generated content will be streamed incrementally.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def get(
        self,
        id: str,
        *,
        api_version: str | None = None,
        stream: Literal[True],
        include_input: bool | Omit = omit,
        last_event_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncStream[InteractionSSEEvent]:
        """
        Retrieves the full details of a single interaction based on its `Interaction.id`.

        Args:
          stream: If set to true, the generated content will be streamed incrementally.

          include_input: If set to true, includes the input in the response.

          last_event_id: Optional. If set, resumes the interaction stream from the next chunk after the event marked by the event id. Can only be used if `stream` is true.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def get(
        self,
        id: str,
        *,
        api_version: str | None = None,
        stream: bool,
        include_input: bool | Omit = omit,
        last_event_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Interaction | AsyncStream[InteractionSSEEvent]:
        """
        Retrieves the full details of a single interaction based on its `Interaction.id`.

        Args:
          stream: If set to true, the generated content will be streamed incrementally.

          include_input: If set to true, includes the input in the response.

          last_event_id: Optional. If set, resumes the interaction stream from the next chunk after the event marked by the event id. Can only be used if `stream` is true.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    async def get(
        self,
        id: str,
        *,
        api_version: str | None = None,
        include_input: bool | Omit = omit,
        last_event_id: str | Omit = omit,
        stream: Literal[False] | Literal[True] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Interaction | AsyncStream[InteractionSSEEvent]:
        if api_version is None:
            api_version = self._client._get_api_version_path_param()
        if not api_version:
            raise ValueError(f"Expected a non-empty value for `api_version` but received {api_version!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            self._client._build_maybe_vertex_path(api_version=api_version, path=f'interactions/{id}'),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "include_input": include_input,
                        "last_event_id": last_event_id,
                        "stream": stream,
                    },
                    interaction_get_params.InteractionGetParams,
                ),
            ),
            cast_to=Interaction,
            stream=stream or False,
            stream_cls=AsyncStream[InteractionSSEEvent],
        )


class InteractionsResourceWithRawResponse:
    def __init__(self, interactions: InteractionsResource) -> None:
        self._interactions = interactions

        self.create = to_raw_response_wrapper(
            interactions.create,
        )
        self.delete = to_raw_response_wrapper(
            interactions.delete,
        )
        self.cancel = to_raw_response_wrapper(
            interactions.cancel,
        )
        self.get = to_raw_response_wrapper(
            interactions.get,
        )


class AsyncInteractionsResourceWithRawResponse:
    def __init__(self, interactions: AsyncInteractionsResource) -> None:
        self._interactions = interactions

        self.create = async_to_raw_response_wrapper(
            interactions.create,
        )
        self.delete = async_to_raw_response_wrapper(
            interactions.delete,
        )
        self.cancel = async_to_raw_response_wrapper(
            interactions.cancel,
        )
        self.get = async_to_raw_response_wrapper(
            interactions.get,
        )


class InteractionsResourceWithStreamingResponse:
    def __init__(self, interactions: InteractionsResource) -> None:
        self._interactions = interactions

        self.create = to_streamed_response_wrapper(
            interactions.create,
        )
        self.delete = to_streamed_response_wrapper(
            interactions.delete,
        )
        self.cancel = to_streamed_response_wrapper(
            interactions.cancel,
        )
        self.get = to_streamed_response_wrapper(
            interactions.get,
        )


class AsyncInteractionsResourceWithStreamingResponse:
    def __init__(self, interactions: AsyncInteractionsResource) -> None:
        self._interactions = interactions

        self.create = async_to_streamed_response_wrapper(
            interactions.create,
        )
        self.delete = async_to_streamed_response_wrapper(
            interactions.delete,
        )
        self.cancel = async_to_streamed_response_wrapper(
            interactions.cancel,
        )
        self.get = async_to_streamed_response_wrapper(
            interactions.get,
        )



# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


"""Tests replay client correctly compares requests ignoring key casing."""

from ... import _replay_api_client
from ... import types


def test_equal_objects_with_same_casing_returns_true():
  obj1 = [{
      'contents': [{'parts': [{'text': 'What is your name?'}], 'role': 'user'}],
      'generationConfig': {'topK': 2.0, 'maxOutputTokens': 3},
  }]
  obj2 = [{
      'contents': [{'parts': [{'text': 'What is your name?'}], 'role': 'user'}],
      'generationConfig': {'topK': 2.0, 'maxOutputTokens': 3},
  }]

  assert _replay_api_client._equals_ignore_key_case(obj1, obj2)


def test_equal_objects_with_different_casing_returns_true():
  obj1 = [{
      'contents': [{'parts': [{'text': 'What is your name?'}], 'role': 'user'}],
      'generationConfig': {'topK': 2.0, 'maxOutputTokens': 3},
  }]
  obj2 = [{
      'contents': [{'parts': [{'text': 'What is your name?'}], 'role': 'user'}],
      'generation_config': {'topK': 2.0, 'max_output_tokens': 3},
  }]

  assert _replay_api_client._equals_ignore_key_case(obj1, obj2)


def test_equal_objects_with_different_nested_casing_returns_true():
  obj1 = [{
      'contents': [{'parts': [{'text': 'What is your name?'}], 'role': 'user'}],
      'generationConfig': {'topK': 2.0, 'maxOutputTokens': 3},
  }]
  obj2 = [{
      'contents': [{'parts': [{'text': 'What is your name?'}], 'role': 'user'}],
      'generationConfig': {'top_k': 2.0, 'maxOutputTokens': 3},
  }]

  assert _replay_api_client._equals_ignore_key_case(obj1, obj2)

def test_equal_int_float_values_returns_true():
  obj1 = [{
      'contents': [{'parts': [{'text': 'What is your name?'}], 'role': 'user'}],
      'generationConfig': {'topK': 2.0, 'maxOutputTokens': 3},
  }]
  obj2 = [{
      'contents': [{'parts': [{'text': 'What is your name?'}], 'role': 'user'}],
      'generation_config': {'topK': 2, 'max_output_tokens': 3},
  }]

  assert _replay_api_client._equals_ignore_key_case(obj1, obj2)


def test_equal_enum_string_values_returns_true():
  obj1 = [{
      'contents': [{'parts': [{'text': 'What is your name?'}], 'role': 'user'}],
      'generationConfig': {
          'topK': 2.0,
          'maxOutputTokens': 3,
          'type': types.Type.STRING,
      },
  }]
  obj2 = [{
      'contents': [{'parts': [{'text': 'What is your name?'}], 'role': 'user'}],
      'generation_config': {
          'topK': 2,
          'max_output_tokens': 3,
          'type': 'STRING',
      },
  }]

  assert _replay_api_client._equals_ignore_key_case(obj1, obj2)


def test_equal_enum_values_returns_true():
  obj1 = [{
      'contents': [{'parts': [{'text': 'What is your name?'}], 'role': 'user'}],
      'generationConfig': {
          'topK': 2.0,
          'maxOutputTokens': 3,
          'type': types.Type.STRING,
      },
  }]
  obj2 = [{
      'contents': [{'parts': [{'text': 'What is your name?'}], 'role': 'user'}],
      'generation_config': {
          'topK': 2,
          'max_output_tokens': 3,
          'type': types.Type.STRING,
      },
  }]

  assert _replay_api_client._equals_ignore_key_case(obj1, obj2)


def test_different_number_value_returns_false():
  obj1 = [{
      'contents': [{'parts': [{'text': 'What is your name?'}], 'role': 'user'}],
      'generationConfig': {'topK': 3, 'maxOutputTokens': 3},
  }]
  obj2 = [{
      'contents': [{'parts': [{'text': 'What is your name?'}], 'role': 'user'}],
      'generation_config': {'topK': 2.0, 'max_output_tokens': 3},
  }]

  assert not _replay_api_client._equals_ignore_key_case(
      obj1, obj2
  )


def test_different_string_value_returns_false():
  obj1 = [{
      'contents': [{'parts': [{'text': 'What is your name?'}], 'role': 'user'}],
      'generationConfig': {'topK': 3, 'maxOutputTokens': 3},
  }]
  obj2 = [{
      'contents': [
          {'parts': [{'text': 'What is your name?'}], 'role': 'model'}
      ],
      'generation_config': {'topK': 3, 'max_output_tokens': 3},
  }]

  assert not _replay_api_client._equals_ignore_key_case(
      obj1, obj2
  )


def test_different_enum_values_returns_false():
  obj1 = [{
      'contents': [{'parts': [{'text': 'What is your name?'}], 'role': 'user'}],
      'generationConfig': {
          'topK': 2.0,
          'maxOutputTokens': 3,
          'type': types.Type.STRING,
      },
  }]
  obj2 = [{
      'contents': [{'parts': [{'text': 'What is your name?'}], 'role': 'user'}],
      'generation_config': {
          'topK': 2,
          'max_output_tokens': 3,
          'type': types.Type.OBJECT,
      },
  }]

  assert not _replay_api_client._equals_ignore_key_case(
      obj1, obj2
  )



# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


"""Tests for Interactions API."""

from ... import Client
from ... import _base_url
from unittest import mock
import pytest
from httpx import Request, Response
from ..._api_client import AsyncHttpxClient, BaseApiClient
from httpx import Client as HTTPClient
import os

ENV_VARS = [
    "GOOGLE_CLOUD_PROJECT",
    "GEMINI_API_KEY",
    "GOOGLE_API_KEY",
    "GOOGLE_CLOUD_LOCATION",
]

@pytest.fixture(autouse=True)
def clear_env_vars(monkeypatch):
    for var in ENV_VARS:
        monkeypatch.delenv(var, raising=False)

def test_interactions_gemini_url(monkeypatch):
    monkeypatch.setenv('GOOGLE_API_KEY', 'test-api-key')
    client = Client()


    with mock.patch.object(HTTPClient, "send") as mock_send:
        mock_send.return_value = Response(200, request=Request('POST', ''))
        client.interactions.create(
            model='gemini-1.5-flash',
            input='Hello',
        )
        mock_send.assert_called_once()
        request = mock_send.call_args[0][0]
        assert str(request.url).endswith('/v1beta/interactions')
        assert request.headers['x-goog-api-key'] == 'test-api-key'


def test_interactions_gemini_no_vertex_auth(monkeypatch):
    monkeypatch.setenv('GOOGLE_API_KEY', 'test-api-key')
    client = Client()

    with (
        mock.patch.object(BaseApiClient, "_access_token") as mock_access_token,
        mock.patch.object(HTTPClient, "send") as mock_send,
    ):
        mock_send.return_value = Response(200, request=Request('POST', ''))
        client.interactions.create(
            model='gemini-1.5-flash',
            input='Hello',
        )
        mock_access_token.assert_not_called()

def test_interactions_gemini_retry(monkeypatch):
    monkeypatch.setenv('GOOGLE_API_KEY', 'test-api-key')
    client = Client()
    client._api_client.max_retries = 2

    with mock.patch.object(HTTPClient, "send") as mock_send:
        mock_send.side_effect = [
            Response(500, request=Request('POST', ''), headers={"retry-after-ms": "1"}),
            Response(500, request=Request('POST', ''), headers={"retry-after-ms": "1"}),
            Response(200, request=Request('POST', '')),
        ]
        client.interactions.create(model='gemini-1.5-flash', input='Hello')
        assert mock_send.call_count == 3

def test_interactions_gemini_extra_headers(monkeypatch):
    monkeypatch.setenv('GOOGLE_API_KEY', 'test-api-key')
    client = Client()

    with mock.patch.object(HTTPClient, "send") as mock_send:
        mock_send.return_value = Response(200, request=Request('POST', ''))
        client.interactions.create(
            model='gemini-1.5-flash',
            input='Hello',
            extra_headers={'X-Custom-Header': 'TestValue'}
        )
        mock_send.assert_called_once()
        request = mock_send.call_args[0][0]
        assert request.headers['x-custom-header'] == 'TestValue'
        assert request.headers['x-goog-api-key'] == 'test-api-key'


def test_interactions_vertex_auth_header():
  from ..._api_client import BaseApiClient
  from ..._interactions._base_client import SyncAPIClient
  from httpx import Client as HTTPClient

  creds = mock.Mock()
  creds.quota_project_id = "test-quota-project"
  client = Client(vertexai=True, project='test-project', location='us-central1', credentials=creds)

  with (
      mock.patch.object(
          BaseApiClient, "_access_token", return_value='fake-vertex-token'
      ) as  mock_access_token,
      mock.patch.object(
          HTTPClient, "send",
          return_value=mock.Mock(),
      ) as mock_send,
  ):

    response = client.interactions.create(
        model='gemini-2.5-flash',
        input='What is the largest planet in our solar system?',
    )

    mock_send.assert_called_once()
    mock_access_token.assert_called_once()
    args, kwargs = mock_send.call_args
    headers = args[0].headers
    assert any(
        key == "authorization" and value == 'Bearer fake-vertex-token'
        for key, value in headers.items())
    assert any(
        key == "x-goog-user-project" and value == 'test-quota-project'
        for key, value in headers.items())

def test_interactions_vertex_key_no_auth_header():
  from ..._api_client import BaseApiClient
  from httpx import Client as HTTPClient

  creds = mock.Mock()
  client = Client(vertexai=True, api_key='test-api-key')

  with (
      mock.patch.object(
          BaseApiClient, "_access_token", return_value='fake-vertex-token'
      ) as  mock_access_token,
      mock.patch.object(
          HTTPClient, "send",
          return_value=mock.Mock(),
      ) as mock_send,
  ):

    response = client.interactions.create(
        model='gemini-2.5-flash',
        input='What is the largest planet in our solar system?',
    )

    mock_send.assert_called_once()
    mock_access_token.assert_not_called()
    args, kwargs = mock_send.call_args
    headers = args[0].headers
    assert any(
        key == "x-goog-api-key" and value == 'test-api-key'
        for key, value in headers.items())

def test_interactions_vertex_url():
    creds = mock.Mock()
    creds.quota_project_id = "test-quota-project"
    client = Client(vertexai=True, project='test-project', location='us-central1', credentials=creds)

    with mock.patch("httpx.Client.send") as mock_send:
        mock_send.return_value = Response(200, request=Request('POST', ''))
        client.interactions.create(
            model='gemini-1.5-flash',
            input='Hello',
        )
        mock_send.assert_called_once()
        request = mock_send.call_args[0][0]
        assert str(request.url) == 'https://us-central1-aiplatform.googleapis.com/v1beta1/projects/test-project/locations/us-central1/interactions'

def test_interactions_vertex_auth_refresh_on_retry():
    from ..._api_client import BaseApiClient
    from httpx import Client as HTTPClient

    creds = mock.Mock()
    creds.quota_project_id = "test-quota-project"
    client = Client(vertexai=True, project='test-project', location='us-central1', credentials=creds)
    client._api_client.max_retries = 2

    token_values = ['token1', 'token2', 'token3']
    token_iter = iter(token_values)

    def get_token():
        return next(token_iter)

    with (
        mock.patch.object(BaseApiClient, "_access_token", side_effect=get_token) as mock_access_token,
        mock.patch.object(HTTPClient, "send") as mock_send,
    ):
        mock_send.side_effect = [
            Response(500, request=Request('POST', ''), headers={"retry-after-ms": "1"}),
            Response(500, request=Request('POST', ''), headers={"retry-after-ms": "1"}),
            Response(200, request=Request('POST', '')),
        ]

        client.interactions.create(model='gemini-1.5-flash', input='Hello')

        assert mock_access_token.call_count == 3
        assert mock_send.call_count == 3
        # Check headers of each call
        for i in range(3):
            headers = mock_send.call_args_list[i][0][0].headers
            assert headers['authorization'] == f'Bearer {token_values[i]}'


def test_interactions_vertex_extra_headers_override():
    from ..._api_client import BaseApiClient
    from httpx import Client as HTTPClient

    creds = mock.Mock()
    creds.quota_project_id = "test-quota-project"
    client = Client(vertexai=True, project='test-project', location='us-central1', credentials=creds)

    with (
        mock.patch.object(BaseApiClient, "_access_token", return_value='default-token') as mock_access_token,
        mock.patch.object(HTTPClient, "send") as mock_send,
    ):
        mock_send.return_value = Response(200, request=Request('POST', ''))

        # Override Authorization
        client.interactions.create(
            model='gemini-1.5-flash',
            input='Hello',
            extra_headers={'Authorization': 'Bearer manual-token'}
        )
        mock_send.assert_called_once()
        headers = mock_send.call_args[0][0].headers
        assert headers['authorization'] == 'Bearer manual-token'
        mock_access_token.assert_not_called() # Should not fetch default token

        mock_send.reset_mock()
        mock_access_token.reset_mock()

        # Provide API Key
        client.interactions.create(
            model='gemini-1.5-flash',
            input='Hello',
            extra_headers={'x-goog-api-key': 'manual-key'}
        )
        mock_send.assert_called_once()
        headers = mock_send.call_args[0][0].headers
        assert headers['x-goog-api-key'] == 'manual-key'
        assert 'authorization' not in headers
        mock_access_token.assert_not_called()

@pytest.mark.asyncio
async def test_async_interactions_gemini_url(monkeypatch):
    monkeypatch.setenv('GOOGLE_API_KEY', 'test-api-key')
    client = Client()

    with mock.patch.object(AsyncHttpxClient, "send") as mock_send:
        mock_send.return_value = Response(200, request=Request('POST', ''))
        await client.aio.interactions.create(
            model='gemini-1.5-flash',
            input='Hello',
        )
        mock_send.assert_called_once()
        request = mock_send.call_args[0][0]
        assert str(request.url).endswith('/v1beta/interactions')
        assert request.headers['x-goog-api-key'] == 'test-api-key'

@pytest.mark.asyncio
async def test_async_interactions_gemini_no_vertex_auth(monkeypatch):
    monkeypatch.setenv('GOOGLE_API_KEY', 'test-api-key')
    client = Client()

    with (
        mock.patch.object(BaseApiClient, "_async_access_token") as mock_access_token,
        mock.patch.object(AsyncHttpxClient, "send") as mock_send,
    ):
        mock_send.return_value = Response(200, request=Request('POST', ''))
        await client.aio.interactions.create(
            model='gemini-1.5-flash',
            input='Hello',
        )
        mock_access_token.assert_not_called()

@pytest.mark.asyncio
async def test_async_interactions_gemini_retry(monkeypatch):
    monkeypatch.setenv('GOOGLE_API_KEY', 'test-api-key')
    client = Client()
    client.aio._api_client.max_retries = 2

    with mock.patch.object(AsyncHttpxClient, "send") as mock_send:
        mock_send.side_effect = [
            Response(500, request=Request('POST', ''), headers={"retry-after-ms": "1"}),
            Response(500, request=Request('POST', ''), headers={"retry-after-ms": "1"}),
            Response(200, request=Request('POST', '')),
        ]
        await client.aio.interactions.create(model='gemini-1.5-flash', input='Hello')
        assert mock_send.call_count == 3

@pytest.mark.asyncio
async def test_async_interactions_gemini_extra_headers(monkeypatch):
    monkeypatch.setenv('GOOGLE_API_KEY', 'test-api-key')
    client = Client()

    with mock.patch.object(AsyncHttpxClient, "send") as mock_send:
        mock_send.return_value = Response(200, request=Request('POST', ''))
        await client.aio.interactions.create(
            model='gemini-1.5-flash',
            input='Hello',
            extra_headers={'X-Custom-Header': 'TestValue'}
        )
        mock_send.assert_called_once()
        request = mock_send.call_args[0][0]
        assert request.headers['x-custom-header'] == 'TestValue'
        assert request.headers['x-goog-api-key'] == 'test-api-key'

@pytest.mark.asyncio
async def test_async_interactions_vertex_auth_header():
  from ..._api_client import BaseApiClient
  from ..._interactions._base_client import SyncAPIClient
  from ..._api_client import AsyncHttpxClient

  creds = mock.Mock()
  creds.quota_project_id = "test-quota-project"
  client = Client(vertexai=True, project='test-project', location='us-central1', credentials=creds)

  with (
      mock.patch.object(
          BaseApiClient, "_async_access_token", return_value='fake-vertex-token'
      ) as  mock_access_token,
      mock.patch.object(
          AsyncHttpxClient, "send",
          return_value=mock.Mock(),
      ) as mock_send,
  ):

    response = await client.aio.interactions.create(
        model='gemini-2.5-flash',
        input='What is the largest planet in our solar system?',
    )

    mock_send.assert_called_once()
    mock_access_token.assert_called_once()
    args, kwargs = mock_send.call_args
    headers = args[0].headers
    assert any(
        key == "authorization" and value == 'Bearer fake-vertex-token'
        for key, value in headers.items())
    assert any(
        key == "x-goog-user-project" and value == 'test-quota-project'
        for key, value in headers.items())

@pytest.mark.asyncio
async def test_async_interactions_vertex_key_no_auth_header():
  from ..._api_client import BaseApiClient
  client = Client(vertexai=True, api_key='test-api-key')

  with (
      mock.patch.object(
          BaseApiClient, "_async_access_token", return_value='fake-vertex-token'
      ) as  mock_access_token,
      mock.patch.object(
          AsyncHttpxClient, "send",
          return_value=mock.Mock(),
      ) as mock_send,
  ):

    response = await client.aio.interactions.create(
        model='gemini-2.5-flash',
        input='What is the largest planet in our solar system?',
    )

    mock_send.assert_called_once()
    mock_access_token.assert_not_called()
    args, kwargs = mock_send.call_args
    headers = args[0].headers
    assert any(
        key == "x-goog-api-key" and value == 'test-api-key'
        for key, value in headers.items())

@pytest.mark.asyncio
async def test_async_interactions_vertex_url():
    from ..._api_client import AsyncHttpxClient
    creds = mock.Mock()
    creds.quota_project_id = "test-quota-project"
    client = Client(vertexai=True, project='test-project', location='us-central1', credentials=creds)

    with mock.patch.object(AsyncHttpxClient, "send") as mock_send:
        mock_send.return_value = Response(200, request=Request('POST', ''))
        await client.aio.interactions.create(
            model='gemini-1.5-flash',
            input='Hello',
        )
        mock_send.assert_called_once()
        request = mock_send.call_args[0][0]
        assert str(request.url) == 'https://us-central1-aiplatform.googleapis.com/v1beta1/projects/test-project/locations/us-central1/interactions'

@pytest.mark.asyncio
async def test_async_interactions_vertex_auth_refresh_on_retry():
    from ..._api_client import BaseApiClient
    from ..._api_client import AsyncHttpxClient

    creds = mock.Mock()
    creds.quota_project_id = "test-quota-project"
    client = Client(vertexai=True, project='test-project', location='us-central1', credentials=creds)
    client.aio._api_client.max_retries = 2

    token_values = ['token1', 'token2', 'token3']
    token_iter = iter(token_values)

    async def get_token():
        return next(token_iter)

    with (
        mock.patch.object(BaseApiClient, "_async_access_token", side_effect=get_token) as mock_access_token,
        mock.patch.object(AsyncHttpxClient, "send") as mock_send,
    ):
        mock_send.side_effect = [
            Response(500, request=Request('POST', ''), headers={"retry-after-ms": "1"}),
            Response(500, request=Request('POST', ''), headers={"retry-after-ms": "1"}),
            Response(200, request=Request('POST', '')),
        ]

        await client.aio.interactions.create(model='gemini-1.5-flash', input='Hello')

        assert mock_access_token.call_count == 3
        assert mock_send.call_count == 3
        for i in range(3):
            headers = mock_send.call_args_list[i][0][0].headers
            assert headers['authorization'] == f'Bearer {token_values[i]}'

@pytest.mark.asyncio
async def test_async_interactions_vertex_extra_headers_override():
    from ..._api_client import BaseApiClient
    from ..._api_client import AsyncHttpxClient

    creds = mock.Mock()
    creds.quota_project_id = "test-quota-project"
    client = Client(vertexai=True, project='test-project', location='us-central1', credentials=creds)

    with (
        mock.patch.object(BaseApiClient, "_async_access_token", return_value='default-token') as mock_access_token,
        mock.patch.object(AsyncHttpxClient, "send") as mock_send,
    ):
        mock_send.return_value = Response(200, request=Request('POST', ''))

        # Override Authorization
        await client.aio.interactions.create(
            model='gemini-1.5-flash',
            input='Hello',
            extra_headers={'Authorization': 'Bearer manual-token'}
        )
        mock_send.assert_called_once()
        headers = mock_send.call_args[0][0].headers
        assert headers['authorization'] == 'Bearer manual-token'
        mock_access_token.assert_not_called()

        mock_send.reset_mock()
        mock_access_token.reset_mock()

        # Provide API Key
        await client.aio.interactions.create(
            model='gemini-1.5-flash',
            input='Hello',
            extra_headers={'x-goog-api-key': 'manual-key'}
        )
        mock_send.assert_called_once()
        headers = mock_send.call_args[0][0].headers
        assert headers['x-goog-api-key'] == 'manual-key'
        assert 'authorization' not in headers
        mock_access_token.assert_not_called()



# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from typing_extensions import Literal, Required, Annotated, TypedDict

from .._types import Base64FileInput
from .._utils import PropertyInfo
from .._models import set_pydantic_config

__all__ = ["ImageContentParam"]


class ImageContentParam(TypedDict, total=False):
    """An image content block."""

    type: Required[Literal["image"]]

    data: Annotated[Union[str, Base64FileInput], PropertyInfo(format="base64")]
    """The image content."""

    mime_type: Literal[
        "image/png", "image/jpeg", "image/webp", "image/heic", "image/heif", "image/gif", "image/bmp", "image/tiff"
    ]
    """The mime type of the image."""

    resolution: Literal["low", "medium", "high", "ultra_high"]
    """The resolution of the media."""

    uri: str
    """The URI of the image."""


set_pydantic_config(ImageContentParam, {"arbitrary_types_allowed": True})



import unittest

import pydantic

from ... import _common


class TestIsDuckTypeOf(unittest.TestCase):

  class FakePydanticModel(pydantic.BaseModel):
    field1: str
    field2: int
    field3: str
    field4: str
    field5: str

  class FakePydanticModelWithLessFields(pydantic.BaseModel):
    field1: str
    field2: int
    field3: str
    field4: str

  def test_is_duck_type_of_true_for_pydantic_object(self):
    obj = self.FakePydanticModel(
        field1="a",
        field2=1,
        field3="b",
        field4="c",
        field5="d",
    )
    self.assertTrue(_common.is_duck_type_of(obj, self.FakePydanticModel))

  def test_is_duck_type_of_true_for_duck_typed_object(self):
    class DuckTypedObject:

      def __init__(self):
        self.field1 = "a"
        self.field2 = 1
        self.field3 = "b"
        self.field4 = "c"
        self.field5 = "d"

    obj = DuckTypedObject()
    self.assertTrue(_common.is_duck_type_of(obj, self.FakePydanticModel))

  def test_is_duck_type_of_false_for_different_many_fields(self):
    class DifferentFieldsObject:

      def __init__(self):
        self.fielda = "a"

    obj = DifferentFieldsObject()
    self.assertFalse(
        _common.is_duck_type_of(obj, self.FakePydanticModel)
    )

  def test_is_duck_type_of_false_for_missing_fields(self):

    obj = self.FakePydanticModelWithLessFields(
        field1="a", field2=1, field3="b", field4="c"
    )
    self.assertFalse(_common.is_duck_type_of(obj, self.FakePydanticModel))

  def test_is_duck_type_of_false_for_dict(self):
    obj = {"field1": "a", "field2": 1}
    self.assertFalse(
        _common.is_duck_type_of(obj, self.FakePydanticModel)
    )

  def test_is_duck_type_of_false_for_non_pydantic_class(self):
    class NonPydanticModel:
      pass

    class SomeObject:
      pass

    obj = SomeObject()
    self.assertFalse(_common.is_duck_type_of(obj, NonPydanticModel))

  def test_is_duck_type_of_true_with_extra_fields(self):
    class ExtraFieldsObject:

      def __init__(self):
        self.field1 = "a"
        self.field2 = 1
        self.field3 = "extra"
        self.field4 = "extra"
        self.field5 = "extra"
        self.field6 = "extra"

    obj = ExtraFieldsObject()
    self.assertTrue(_common.is_duck_type_of(obj, self.FakePydanticModel))


if __name__ == "__main__":
  unittest.main()



# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


"""Test for the code sample for Gemini text-only request."""

from .. import pytest_helper

pytestmark = pytest_helper.setup(file=__file__)


def test_sample(client):
  # [START generativeaionvertexai_gemini_text_only]
  response = client.models.generate_content(
      model="gemini-2.5-flash",
      contents=(
          "What's a good name for a flower shop that specializes in selling"
          " bouquets of dried flowers?"
      ),
  )
  print(response.text)
  # [END generativeaionvertexai_gemini_text_only]



# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import time
from typing import TYPE_CHECKING

import anyio

if TYPE_CHECKING:
    from ._client import GeminiNextGenAPIClient, AsyncGeminiNextGenAPIClient


class SyncAPIResource:
    _client: GeminiNextGenAPIClient

    def __init__(self, client: GeminiNextGenAPIClient) -> None:
        self._client = client
        self._get = client.get
        self._post = client.post
        self._patch = client.patch
        self._put = client.put
        self._delete = client.delete
        self._get_api_list = client.get_api_list

    def _sleep(self, seconds: float) -> None:
        time.sleep(seconds)


class AsyncAPIResource:
    _client: AsyncGeminiNextGenAPIClient

    def __init__(self, client: AsyncGeminiNextGenAPIClient) -> None:
        self._client = client
        self._get = client.get
        self._post = client.post
        self._patch = client.patch
        self._put = client.put
        self._delete = client.delete
        self._get_api_list = client.get_api_list

    async def _sleep(self, seconds: float) -> None:
        await anyio.sleep(seconds)



# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

"""Tests for live response handling."""
import json
from typing import cast
from unittest import mock
from unittest.mock import AsyncMock

import pytest

from ... import _api_client as api_client
from ... import _common
from ... import Client
from ... import client as gl_client
from ... import live
from ... import types


def mock_api_client(vertexai=False):
  """Creates a mock BaseApiClient."""
  mock_client = mock.MagicMock(spec=gl_client.BaseApiClient)
  if not vertexai:
    mock_client.api_key = 'TEST_API_KEY'
    mock_client.location = None
    mock_client.project = None
  else:
    mock_client.api_key = None
    mock_client.location = 'us-central1'
    mock_client.project = 'test_project'

  mock_client._host = lambda: 'test_host'
  mock_client._http_options = types.HttpOptions.model_validate(
      {'headers': {}}
  )
  mock_client.vertexai = vertexai
  return mock_client


@pytest.fixture
def mock_websocket():
  """Provides a mock websocket connection."""
  # Use live.ClientConnection if that's the specific type hint in AsyncSession
  websocket = AsyncMock(spec=live.ClientConnection)
  websocket.send = AsyncMock()
  # Set default recv value, will be overridden in the test
  websocket.recv = AsyncMock(return_value='{}')
  websocket.close = AsyncMock()
  return websocket


@pytest.mark.parametrize('vertexai', [True, False])
@pytest.mark.asyncio
async def test_receive_server_content(mock_websocket, vertexai):

  raw_response_json = json.dumps({
      "usageMetadata": {
          "promptTokenCount": 15,
          "responseTokenCount": 25,
          "candidatesTokenCount": 50,
          "totalTokenCount": 200,
          "responseTokensDetails": [
              {
                  "tokenCount": 20,
                  "modality": "TEXT",
              }
          ],
          "candidatesTokensDetails": [
              {
                  "tokenCount": 10,
                  "modality": "TEXT",
              }
          ],
      },
      "serverContent": {
          "modelTurn": {
              "parts": [{"text": "This is a simple response."}]
          },
          "turnComplete": True,
          "groundingMetadata": {
              "web_search_queries": ["test query"],
              "groundingChunks": [{
                  "web": {
                      "domain": "google.com",
                      "title": "Search results",
                  }
              }]
          }
      }
  })
  mock_websocket.recv.return_value = raw_response_json

  session = live.AsyncSession(
      api_client=mock_api_client(vertexai=vertexai), websocket=mock_websocket
  )
  result = await session._receive()

  # Assert the results
  assert isinstance(result, types.LiveServerMessage)

  assert (
      result.server_content.model_turn.parts[0].text
      == "This is a simple response."
  )
  assert result.server_content.turn_complete
  assert result.server_content.grounding_metadata.web_search_queries == ["test query"]
  assert result.server_content.grounding_metadata.grounding_chunks[0].web.domain == "google.com"
  assert result.server_content.grounding_metadata.grounding_chunks[0].web.title == "Search results"
  # Verify usageMetadata was parsed
  assert isinstance(result.usage_metadata, types.UsageMetadata)
  assert result.usage_metadata.prompt_token_count == 15
  assert result.usage_metadata.total_token_count == 200
  if not vertexai:
    assert result.usage_metadata.response_token_count == 25
    assert result.usage_metadata.response_tokens_details[0].token_count == 20
  else:
    # VertexAI maps candidatesTokenCount to responseTokenCount and maps
    # candidatesTokensDetails to responseTokensDetails.
    assert result.usage_metadata.response_token_count == 50
    assert result.usage_metadata.response_tokens_details[0].token_count == 10

@pytest.mark.parametrize('vertexai', [True, False])
@pytest.mark.asyncio
async def test_receive_server_content_with_turn_reason(mock_websocket, vertexai):
  """Tests parsing of LiveServerContent with turn_complete_reason and waiting_for_input."""

  raw_response_json = json.dumps({
      "serverContent": {
          "modelTurn": {
              "parts": [{"text": "Please provide more details."}]
          },
          "turnComplete": True,
          "turnCompleteReason": "NEED_MORE_INPUT",
          "waitingForInput": True
      }
  })
  mock_websocket.recv.return_value = raw_response_json

  session = live.AsyncSession(
      api_client=mock_api_client(vertexai=vertexai), websocket=mock_websocket
  )
  result = await session._receive()

  # Assert the results
  assert isinstance(result, types.LiveServerMessage)
  assert result.server_content is not None

  assert result.server_content.model_turn.parts[0].text == "Please provide more details."
  assert result.server_content.turn_complete is True
  assert result.server_content.turn_complete_reason == types.TurnCompleteReason.NEED_MORE_INPUT
  assert result.server_content.waiting_for_input is True



# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#


"""Test invoke_function_from_dict_args."""

from typing import Union
import pydantic
import pytest
import sys
from ... import errors
from ..._extra_utils import invoke_function_from_dict_args


def test_builtin_primitive_types():
  def func_under_test(x: int, y: float, z: str, w: bool):
    return {
        'x': x + 1,
        'y': y + 1.0,
        'z': z + '1.0',
        'w': not w,
    }

  original_args = {
      'x': 1,
      'y': 1.0,
      'z': '1.0',
      'w': True,
  }
  expected_response = {
      'x': 2,
      'y': 2.0,
      'z': '1.01.0',
      'w': False,
  }

  actual_response = invoke_function_from_dict_args(
      original_args, func_under_test
  )

  assert actual_response == expected_response


@pytest.mark.skipif(
    sys.version_info < (3, 10),
    reason='| is only supported in Python 3.9 and above.',
)
def test_builtin_compound_types():
  def func_under_test(x: list[int], y: dict[str, float]):
    return {
        'new_x': x + [3],
        'new_y': y | {'key3': 3.0},
    }

  original_args = {
      'x': [1, 2],
      'y': {'key1': 1.0, 'key2': 2.0},
  }
  expected_response = {
      'new_x': [1, 2, 3],
      'new_y': {'key1': 1.0, 'key2': 2.0, 'key3': 3.0},
  }

  actual_response = invoke_function_from_dict_args(
      original_args, func_under_test
  )

  assert actual_response == expected_response


def test_nested_pydantic_model():
  class SimpleModel(pydantic.BaseModel):
    key1_simple: int
    key2_simple: float

  class ComplexModel(pydantic.BaseModel):
    key1_complex: SimpleModel
    key2_complex: list[SimpleModel]
    key3_complex: dict[str, SimpleModel]

  def func_under_test(x: ComplexModel):
    y = x.model_copy()
    y.key1_complex.key1_simple += 1
    y.key1_complex.key2_simple += 1.0
    for simple_model in y.key2_complex:
      simple_model.key1_simple += 1
      simple_model.key2_simple += 1.0
    for simple_model in y.key3_complex.values():
      simple_model.key1_simple += 1
      simple_model.key2_simple += 1.0
    return y

  original_args = {
      'x': {
          'key1_complex': {'key1_simple': 1, 'key2_simple': 1.0},
          'key2_complex': [
              {'key1_simple': 2, 'key2_simple': 2.0},
              {'key1_simple': 3, 'key2_simple': 3.0},
          ],
          'key3_complex': {
              'key1_simple': {'key1_simple': 4, 'key2_simple': 4.0},
              'key2_simple': {'key1_simple': 5, 'key2_simple': 5.0},
          },
      }
  }
  expected_response = {
      'key1_complex': {'key1_simple': 2, 'key2_simple': 2.0},
      'key2_complex': [
          {'key1_simple': 3, 'key2_simple': 3.0},
          {'key1_simple': 4, 'key2_simple': 4.0},
      ],
      'key3_complex': {
          'key1_simple': {'key1_simple': 5, 'key2_simple': 5.0},
          'key2_simple': {'key1_simple': 6, 'key2_simple': 6.0},
      },
  }

  actual_response = invoke_function_from_dict_args(
      original_args, func_under_test
  )

  assert isinstance(actual_response, ComplexModel)
  assert isinstance(actual_response.key1_complex, SimpleModel)
  assert isinstance(actual_response.key2_complex[0], SimpleModel)
  assert isinstance(actual_response.key2_complex[1], SimpleModel)
  assert isinstance(actual_response.key3_complex['key1_simple'], SimpleModel)
  assert isinstance(actual_response.key3_complex['key2_simple'], SimpleModel)
  assert actual_response.model_dump() == expected_response


def test_pydantic_model_in_list_union_type():
  class SimpleModel(pydantic.BaseModel):
    key1_simple: int
    key2_simple: float

  def func_under_test(x: list[Union[int, SimpleModel]]):
    result = []
    for item in x:
      if isinstance(item, int):
        result.append(item + 1)
      elif isinstance(item, SimpleModel):
        result.append(item.model_copy())
        result[-1].key1_simple += 1
        result[-1].key2_simple += 1.0
      else:
        raise ValueError('Unsupported type: %s' % type(item))
    return result

  expected_response = [2, {'key1_simple': 2, 'key2_simple': 2.0}]
  original_args = {'x': [1, {'key1_simple': 1, 'key2_simple': 1.0}]}
  actual_response = invoke_function_from_dict_args(
      original_args, func_under_test
  )
  assert isinstance(actual_response, list)
  assert len(actual_response) == 2
  assert isinstance(actual_response[0], int)
  assert isinstance(actual_response[1], SimpleModel)
  assert actual_response[0] == expected_response[0]
  assert actual_response[1].model_dump() == expected_response[1]


def test_unknown_pydantic_model_argument():
  class SimpleModel(pydantic.BaseModel):
    key1_simple: int
    key2_simple: float

  def func_under_test(x: SimpleModel):
    return x.model_copy()

  original_args = {'x': {'key3_simple': 1, 'key2_simple': 1.0}}

  with pytest.raises(errors.UnknownFunctionCallArgumentError):
    invoke_function_from_dict_args(original_args, func_under_test)


def test_unknown_pydantic_model_argument_with_union_type():
  class SimpleModel1(pydantic.BaseModel):
    key1_simple: int
    key2_simple: float

  class SimpleModel2(pydantic.BaseModel):
    key3_simple: str
    key4_simple: float

  def func_under_test(x: Union[SimpleModel1, SimpleModel2]):
    return x.model_copy()

  original_args = {'x': {'key5_simple': 1, 'key4_simple': 1.0}}

  with pytest.raises(errors.UnknownFunctionCallArgumentError):
    invoke_function_from_dict_args(original_args, func_under_test)


def test_unknown_pydantic_model_argument_with_union_type_and_builtin_type():
  class SimpleModel1(pydantic.BaseModel):
    key1_simple: int
    key2_simple: float

  def func_under_test(x: Union[SimpleModel1, int]):
    return x.model_copy()

  original_args = {'x': {'key5_simple': 1, 'key4_simple': 1.0}}

  with pytest.raises(errors.UnknownFunctionCallArgumentError):
    invoke_function_from_dict_args(original_args, func_under_test)


def test_incompatible_value_and_annotation():
  def func_under_test(x: int):
    return x + 1

  with pytest.raises(errors.UnknownFunctionCallArgumentError):
    invoke_function_from_dict_args({'x': {'k': 'v'}}, func_under_test)
  with pytest.raises(errors.UnknownFunctionCallArgumentError):
    invoke_function_from_dict_args({'x': 'a'}, func_under_test)
  with pytest.raises(errors.UnknownFunctionCallArgumentError):
    invoke_function_from_dict_args({'x': []}, func_under_test)
  with pytest.raises(errors.UnknownFunctionCallArgumentError):
    invoke_function_from_dict_args({'x': 1.0}, func_under_test)
  with pytest.raises(errors.UnknownFunctionCallArgumentError):
    invoke_function_from_dict_args({'x': {}}, func_under_test)


def test_function_invocation_error():
  def func_under_test(x: int):
    return x / 0

  with pytest.raises(errors.FunctionInvocationError):
    invoke_function_from_dict_args({'x': 1}, func_under_test)
